@@ -18,7 +18,9 @@ use failures qw{
 use Hash::Ordered;
 use PDL::Basic qw(sequence);
 use PDL::Core qw(pdl null);
-use List::AllUtils qw(each_arrayref pairgrep pairkeys pairmap pairwise);
+use List::AllUtils qw(
+  each_arrayref pairgrep pairkeys pairmap pairwise reduce
+);
 use List::MoreUtils 0.423;
 use PDL::Primitive ();
 use PDL::Factor    ();
@@ -30,7 +32,9 @@ use Sereal::Decoder 4.005;
 use Sereal::Encoder 4.005;
 use Text::Table::Tiny;
 use Type::Params;
-use Types::Standard qw(Any ArrayRef CodeRef CycleTuple HashRef Maybe Str);
+use Types::Standard qw(
+  Any ArrayRef CodeRef Enum CycleTuple HashRef Maybe Str
+);
 use Types::PDL qw(Piddle Piddle1D);
 
 use Data::Frame::Column::Helper;
@@ -158,14 +162,15 @@ sub BUILD {
 method _format_cell ($col, $ridx,
                      :$na='BAD', :$isbad=$col->isbad)
 {
+    my $is_numeric = $self->_is_numeric_column($col);
     if ($isbad->at($ridx)) {
-        return $na;
+        return $is_numeric ? $na : "<$na>";
     }
 
     if ( $col->$_DOES('PDL::DateTime') ) {
         return $col->dt_at($ridx);
     }
-    elsif ( $self->_is_numeric_column($col) ) {
+    elsif ( $is_numeric ) {
         if ( $col->type >= PDL::float ) {
 
             # This is to fix some float precision problem with perl
@@ -429,7 +434,7 @@ method column($colname) {
 			msg => "column $colname does not exist",
 			trace => failure->croak_trace,
 		}) unless $self->exists( $colname );
-	return $self->_columns->get( $colname );
+	return $self->_columns->get($colname);
 }
 
 # supports negative indices
@@ -818,17 +823,30 @@ method sorti ($by_columns, $ascending=true) {
 }
 
 
-method _serialize_row ($i) {
-    state $sereal = Sereal::Encoder->new();
-    my @row_data = map { $self->column($_)->at($i) } @{ $self->column_names };
+my $sereal = Sereal::Encoder->new();
+
+sub _serialize_row {
+    my ($self, $i, $columns, $columns_isbad) = @_;
+
+    my @row_data =
+      map {
+        my $isbad = $columns_isbad->[$_];
+        ( defined $isbad and $isbad->at($i) )
+          ? ( 1, undef )
+          : ( 0, $columns->[$_]->at($i) );
+      } ( 0 .. $#$columns );
     return $sereal->encode( \@row_data );
 }
 
 method uniq () {
     my %uniq;
     my @uniq_ridx;
+
+    my @columns = map { $self->column($_) } @{ $self->column_names };
+    my @columns_isbad = map { $_->badflag ? $_->isbad : undef } @columns;
+
     for my $i ( 0 .. $self->nrow - 1 ) {
-        my $key = $self->_serialize_row($i);
+        my $key = $self->_serialize_row($i, \@columns, \@columns_isbad);
         unless ( exists $uniq{$key} ) {
             $uniq{$key} = 1;
             push @uniq_ridx, $i;
@@ -839,10 +857,13 @@ method uniq () {
 
 
 method id () {
+    my @columns = map { $self->column($_) } @{ $self->column_names };
+    my @columns_isbad = map { $_->badflag ? $_->isbad : undef } @columns;
+
     my %uniq_serialized;
     my @uniq_rindices;
     for my $ridx ( 0 .. $self->nrow - 1 ) {
-        my $key = $self->_serialize_row($ridx);
+        my $key = $self->_serialize_row($ridx, \@columns, \@columns_isbad);
         if ( not exists $uniq_serialized{$key} ) {
             $uniq_serialized{$key} = [];
             push @uniq_rindices, $ridx;
@@ -905,6 +926,27 @@ sub _is_numeric_column {
     return !is_discrete($piddle);
 }
 
+
+method drop_bad (:$how='any') {
+    state $check = Type::Params::compile( Enum [qw(all any)] );
+    ($how) = $check->($how);
+
+    my $f = sub {
+        my ($op) = @_;
+        return reduce {
+            my $col = $self->column($b);
+            $col->badflag ? $op->( $a, $col->isbad ) : $a;
+        } PDL::Core::zeros( $self->nrow ), $self->names->flatten;
+    };
+
+    my $isbad =
+        $how eq 'any'
+      ? $f->( sub { $_[0] | $_[1] } )
+      : $f->( sub { $_[0] & $_[1] } );
+    return $self->select_rows( PDL::Primitive::which( !$isbad ) );
+}
+
+
 method _compare ($other, $mode) {
     my $class = ref($self);
 
@@ -1050,7 +1092,7 @@ Data::Frame - data frame implementation
 
 =head1 VERSION
 
-version 0.0050_02
+version 0.0052_01
 
 =head1 STATUS
 
@@ -1607,6 +1649,12 @@ This method is internally used by the C<.=> operation, below are same,
 
     is_numeric_column($column_name_or_idx)
 
+=head2 drop_bad
+
+    drop_bad(:$how='any')
+
+Returns a new data frame with rows with BAD values dropped.
+
 =head1 MISCELLANEOUS FEATURES
 
 =head2 Serialization
