@@ -7,7 +7,7 @@ use warnings;
 
 use PDL::Lite ();   # PDL::Lite is the minimal to get PDL work
 use PDL::Core qw(pdl);
-use PDL::Primitive qw(which whichND);
+use PDL::Primitive qw(which);
 
 use Ref::Util qw(is_plain_arrayref);
 use Safe::Isa;
@@ -113,7 +113,7 @@ sub initialize {
 # code modified from <https://metacpan.org/pod/Hash::Path>
 sub _array_get {
     my ( $self, $array, $indices ) = @_;
-    return $array unless scalar @$indices;
+
     my $return_value = $array->[ $indices->[0] ];
     for ( 1 .. $#$indices ) {
         $return_value = $return_value->[ $indices->[$_] ];
@@ -168,20 +168,30 @@ sub glue {
 
 
 sub uniq {
-    my $self  = shift;
+    my ($self) = @_;
     my $class = ref($self);
 
     my @uniq = List::AllUtils::uniq( grep { defined $_ }
           @{ $self->_effective_internal } );
-    my $new = $class->new( \@uniq );
-    return $new;
+    return $class->new( \@uniq );
+}
+
+## Please see file perltidy.ERR
+sub uniqind {
+    my ($self) = @_;
+
+    my $effective_internal = $self->_effective_internal;
+    my @uniqind = List::AllUtils::uniq_by { $effective_internal->[$_] }
+        grep { defined $effective_internal->[$_] }
+            ( 0 .. $#$effective_internal );
+    return pdl( \@uniqind );
 }
 
 
 sub sever {
     my ($self) = @_;
 
-    $self->_internal( [ map { $_ // 'BAD' } @{ $self->_effective_internal } ] );
+    $self->_internal( $self->_effective_internal );
     my $p = PDL->sequence( $self->dims );
     $p = $p->setbadif( $self->isbad ) if $self->badflag;
     $self->{PDL} = $p;
@@ -211,6 +221,10 @@ sub at {
 sub unpdl {
     my $self = shift;
 
+    if ($self->ndims == 1) {    # shortcut for 1D for performance
+        return [ $self->list ];
+    }
+
     state $rmap = sub {
         my ( $x, $f ) = @_;
         is_plain_arrayref($x)
@@ -230,13 +244,16 @@ sub unpdl {
 sub list {
     my ($self) = @_;
 
-    my @data     = $self->{PDL}->list;
     my $internal = $self->_internal;
+    my @list = do {
+        no warnings 'numeric';
+        map { $internal->[$_] } $self->{PDL}->list;
+    };
     if ($self->badflag) {
-        return (map { $_ eq 'BAD' ? 'BAD' : $internal->[$_] } @data);
-    } else {
-        return (map { $internal->[$_] } @data);
+        my @bad_indices = which($self->isbad)->list;
+        @list[@bad_indices] = (('BAD') x @bad_indices);
     }
+    return @list;
 }
 
 
@@ -245,8 +262,10 @@ sub copy {
 
     my $new = PDL::SV->new( [] );
     $new->{PDL} = PDL->sequence( $self->dims );
-    $new->{PDL} = $new->{PDL}->setbadif( $self->isbad ) if $self->badflag;
-    $new->_internal( [ map { $_ // 'BAD' } @{ $self->_effective_internal } ] );
+    $new->_internal( $self->_effective_internal );
+    if ( $self->badflag ) {
+        $new->{PDL} = $new->{PDL}->setbadif( $self->isbad );
+    }
     return $new;
 }
 
@@ -321,9 +340,15 @@ sub _effective_internal {
     my ($self) = @_;
 
     my $internal = $self->_internal;
-    my $rslt =
-      [ map { $_ eq 'BAD' ? undef : $internal->[$_] } ( $self->{PDL}->list ) ];
-    return $rslt;
+    my @indices = $self->{PDL}->list;
+
+    no warnings 'numeric';
+    my @rslt = @$internal[@indices];
+    if ($self->badflag) {
+        my @isbad = which($self->isbad)->list;
+        @rslt[@isbad] = ((undef) x @isbad);
+    }
+    return \@rslt;
 }
 
 sub _compare {
@@ -404,7 +429,7 @@ PDL::SV - PDL subclass for keeping scalar data (like strings)
 
 =head1 VERSION
 
-version 0.0050_02
+version 0.0052_01
 
 =head1 SYNOPSIS
 
@@ -447,6 +472,10 @@ For now it only supports 1D PDL::SV piddles, and C<$dim> has to be C<0>.
 
 BAD values are not considered unique and are ignored.
 
+=head2 uniqind()
+
+Return the indices of all uniq elements of a piddle.
+
 =head2 sever
 
     sever()
